unit UAutoCAD;

interface

uses
  Windows, SysUtils, Messages, Variants, Classes, Registry, ComObj, ActiveX,
  Graphics, AutoCAD_TLB;

resourcestring
  rsDefClassName = 'AutoCAD.Application';

type
  TPoint3D = record
    X, Y, Z: Double;
  end;

type
  AcOSnapMode = TOleEnum;
const
  AcNone = $00000000;
  AcEndPoint = $00000001;
  AcMidPoint = $00000002;
  AcCenter = $00000004;
  AcNode = $00000008;
  AcQuadrant = $00000010;
  AcIntersection = $00000020;
  AcInsertion = $00000040;
  AcPerpendicular = $00000080;
  AcTangent = $00000100;
  AcNarest = $00000200;
  AcQuick = $00000400;
  AcApparentIntersection = $00000800;
  AcExtension = $00001000;
  AcParallel = $00002000;
  AcOSnapAll = $00003FFF;
  AcOSnapOff = $00004000;

type
  AcSnapMode = TOleEnum;
const
  AcTurnsOff = $00000000;
  AcAutoSnapMarker = $00000001;
  AcAutoSnapTooltips = $00000002;
  AcAutoSnapMagnet = $00000004;
  AcPolarTracking = $00000008;
  AcObjectSnapTracking = $00000010;
  AcToolTipsOn = $00000020;
  
type
  AcAttributeMode = TOleEnum;
const
  acAttributeModeNormal = $00000000;
  acAttributeModeInvisible = $00000001;
  acAttributeModeConstant = $00000002;
  acAttributeModeVerify = $00000004;
  acAttributeModePreset = $00000008;

type
  TLayerState = (lsLayerOn, lsFreeze, lsLock, lsPlottable);
  TLayerStates = set of TLayerState;
  TLongWordSize = 0..31;
  TWorkSpace = (wsModelSpace, wsPaperSpace);

  procedure AcConnect(const acClassName: String; const acNewInstance: Boolean;
      const acVisible: Boolean);
  procedure AcShow(WndState: AcWindowState = acMax);
  procedure AcHide;
  procedure AcRelease;
  function EnumAutoCAD(Items: TStrings): Boolean;
  function GetCurrAutoCADClassName: String;
  function AutoCADFamilyNameToClassName(const AFamilyName: String): String;
  function EnumAutoCADHardcopy(Items: TStrings): Boolean;
  //
  function EnumAutoCADDocuments(Items: TStrings): Integer;
  function EnumAutoCADLayers(Items: TStrings): Integer;
  function EnumAutoCADTextStyles(Items: TStrings): Integer;
  function EnumAutoCADDimStyles(Items: TStrings): Integer;
  function EnumAutoCADLinetypes(Items: TStrings): Integer;
  function EnumBlockNames(Items: TStrings; IncludeSpecial: Boolean = False): Integer;
  //
  function LayerExist(const ALayerName: String): Boolean;
  function LinetypeExist(const ALinetypeName: String): Boolean;
  //
  procedure SetActiveLayer(AName: String);
  //
  function Point3D(X, Y: Double; Z: Double = 0): TPoint3D;
  function AcadPoint(X, Y: Double; Z: Double = 0): OleVariant; overload;
  function AcadPoint(APoint: TPoint3D): OleVariant; overload;
  function AcadColorToColor(AColor: AcColor): TColor;
  //
  function IsBit(const Value: LongWord; const n: TLongWordSize): Boolean;
  procedure SetBit(var Value: LongWord; const n: TLongWordSize);
  procedure ResetBit(var Value: LongWord; const n: Byte);
  //
  procedure ExtractBlkRefAttr(ABlkRef: OleVariant; AFileName: string); overload;
  procedure ExtractBlkRefAttr(ABlkRef: OleVariant; Items: TStrings); overload;
  procedure ExtractBlockAttr(AFileName: string; Space: TWorkSpace = wsModelSpace);

type
  TAcadColorRec = record
    I: Byte;
    R: Byte;
    G: Byte;
    B: Byte;
  end;

const
  AcadColorIndex: array[0..255] of TAcadColorRec = (
    (I:  0; R:255; G:  0; B:  0), (I:  1; R:255; G:  0; B:  0),
    (I:  2; R:255; G:255; B:  0), (I:  3; R:  0; G:255; B:  0),
    (I:  4; R:  0; G:255; B:255), (I:  5; R:  0; G:  0; B:255),
    (I:  6; R:255; G:  0; B:255), (I:  7; R:255; G:255; B:255),
    (I:  8; R:128; G:128; B:128), (I:  9; R:192; G:192; B:192),
    (I: 10; R:255; G:  0; B:  0), (I: 11; R:255; G:127; B:127),
    (I: 12; R:204; G:  0; B:  0), (I: 13; R:204; G:102; B:102),
    (I: 14; R:153; G:  0; B:  0), (I: 15; R:153; G: 76; B: 76),
    (I: 16; R:127; G:  0; B:  0), (I: 17; R:127; G: 63; B: 63),
    (I: 18; R: 76; G:  0; B:  0), (I: 19; R: 76; G: 38; B: 38),
    (I: 20; R:255; G: 63; B:  0), (I: 21; R:255; G:159; B:127),
    (I: 22; R:204; G: 51; B:  0), (I: 23; R:204; G:127; B:102),
    (I: 24; R:153; G: 38; B:  0), (I: 25; R:153; G: 95; B: 76),
    (I: 26; R:127; G: 31; B:  0), (I: 27; R:127; G: 79; B: 63),
    (I: 28; R: 76; G: 19; B:  0), (I: 29; R: 76; G: 47; B: 38),
    (I: 30; R:255; G:127; B:  0), (I: 31; R:255; G:191; B:127),
    (I: 32; R:204; G:102; B:  0), (I: 33; R:204; G:153; B:102),
    (I: 34; R:153; G: 76; B:  0), (I: 35; R:153; G:114; B: 76),
    (I: 36; R:127; G: 63; B:  0), (I: 37; R:127; G: 95; B: 63),
    (I: 38; R: 76; G: 38; B:  0), (I: 39; R: 76; G: 57; B: 38),
    (I: 40; R:255; G:191; B:  0), (I: 41; R:255; G:223; B:127),
    (I: 42; R:204; G:153; B:  0), (I: 43; R:204; G:178; B:102),
    (I: 44; R:153; G:114; B:  0), (I: 45; R:153; G:133; B: 76),
    (I: 46; R:127; G: 95; B:  0), (I: 47; R:127; G:111; B: 63),
    (I: 48; R: 76; G: 57; B:  0), (I: 49; R: 76; G: 66; B: 38),
    (I: 50; R:255; G:255; B:  0), (I: 51; R:255; G:255; B:127),
    (I: 52; R:204; G:204; B:  0), (I: 53; R:204; G:204; B:102),
    (I: 54; R:153; G:153; B:  0), (I: 55; R:153; G:153; B: 76),
    (I: 56; R:127; G:127; B:  0), (I: 57; R:127; G:127; B: 63),
    (I: 58; R: 76; G: 76; B:  0), (I: 59; R: 76; G: 76; B: 38),
    (I: 60; R:191; G:255; B:  0), (I: 61; R:223; G:255; B:127),
    (I: 62; R:153; G:204; B:  0), (I: 63; R:178; G:204; B:102),
    (I: 64; R:114; G:153; B:  0), (I: 65; R:133; G:153; B: 76),
    (I: 66; R: 95; G:127; B:  0), (I: 67; R:111; G:127; B: 63),
    (I: 68; R: 57; G: 76; B:  0), (I: 69; R: 66; G: 76; B: 38),
    (I: 70; R:127; G:255; B:  0), (I: 71; R:191; G:255; B:127),
    (I: 72; R:102; G:204; B:  0), (I: 73; R:153; G:204; B:102),
    (I: 74; R: 76; G:153; B:  0), (I: 75; R:114; G:153; B: 76),
    (I: 76; R: 63; G:127; B:  0), (I: 77; R: 95; G:127; B: 63),
    (I: 78; R: 38; G: 76; B:  0), (I: 79; R: 57; G: 76; B: 38),
    (I: 80; R: 63; G:255; B:  0), (I: 81; R:159; G:255; B:127),
    (I: 82; R: 51; G:204; B:  0), (I: 83; R:127; G:204; B:102),
    (I: 84; R: 38; G:153; B:  0), (I: 85; R: 95; G:153; B: 76),
    (I: 86; R: 31; G:127; B:  0), (I: 87; R: 79; G:127; B: 63),
    (I: 88; R: 19; G: 76; B:  0), (I: 89; R: 47; G: 76; B: 38),
    (I: 90; R:  0; G:255; B:  0), (I: 91; R:127; G:255; B:127),
    (I: 92; R:  0; G:204; B:  0), (I: 93; R:102; G:204; B:102),
    (I: 94; R:  0; G:153; B:  0), (I: 95; R: 76; G:153; B: 76),
    (I: 96; R:  0; G:127; B:  0), (I: 97; R: 63; G:127; B: 63),
    (I: 98; R:  0; G: 76; B:  0), (I: 99; R: 38; G: 76; B: 38),
    (I:100; R:  0; G:255; B: 63), (I:101; R:127; G:255; B:159),
    (I:102; R:  0; G:204; B: 51), (I:103; R:102; G:204; B:127),
    (I:104; R:  0; G:153; B: 38), (I:105; R: 76; G:153; B: 95),
    (I:106; R:  0; G:127; B: 31), (I:107; R: 63; G:127; B: 79),
    (I:108; R:  0; G: 76; B: 19), (I:109; R: 38; G: 76; B: 47),
    (I:110; R:  0; G:255; B:127), (I:111; R:127; G:255; B:191),
    (I:112; R:  0; G:204; B:102), (I:113; R:102; G:204; B:153),
    (I:114; R:  0; G:153; B: 76), (I:115; R: 76; G:153; B:114),
    (I:116; R:  0; G:127; B: 63), (I:117; R: 63; G:127; B: 95),
    (I:118; R:  0; G: 76; B: 38), (I:119; R: 38; G: 76; B: 57),
    (I:120; R:  0; G:255; B:191), (I:121; R:127; G:255; B:223),
    (I:122; R:  0; G:204; B:153), (I:123; R:102; G:204; B:178),
    (I:124; R:  0; G:153; B:114), (I:125; R: 76; G:153; B:133),
    (I:126; R:  0; G:127; B: 95), (I:127; R: 63; G:127; B:111),
    (I:128; R:  0; G: 76; B: 57), (I:129; R: 38; G: 76; B: 66),
    (I:130; R:  0; G:255; B:255), (I:131; R:127; G:255; B:255),
    (I:132; R:  0; G:204; B:204), (I:133; R:102; G:204; B:204),
    (I:134; R:  0; G:153; B:153), (I:135; R: 76; G:153; B:153),
    (I:136; R:  0; G:127; B:127), (I:137; R: 63; G:127; B:127),
    (I:138; R:  0; G: 76; B: 76), (I:139; R: 38; G: 76; B: 76),
    (I:140; R:  0; G:191; B:255), (I:141; R:127; G:223; B:255),
    (I:142; R:  0; G:153; B:204), (I:143; R:102; G:178; B:204),
    (I:144; R:  0; G:114; B:153), (I:145; R: 76; G:133; B:153),
    (I:146; R:  0; G: 95; B:127), (I:147; R: 63; G:111; B:127),
    (I:148; R:  0; G: 57; B: 76), (I:149; R: 38; G: 66; B: 76),
    (I:150; R:  0; G:127; B:255), (I:151; R:127; G:191; B:255),
    (I:152; R:  0; G:102; B:204), (I:153; R:102; G:153; B:204),
    (I:154; R:  0; G: 76; B:153), (I:155; R: 76; G:114; B:153),
    (I:156; R:  0; G: 63; B:127), (I:157; R: 63; G: 95; B:127),
    (I:158; R:  0; G: 38; B: 76), (I:159; R: 38; G: 57; B: 76),
    (I:160; R:  0; G: 63; B:255), (I:161; R:127; G:159; B:255),
    (I:162; R:  0; G: 51; B:204), (I:163; R:102; G:127; B:204),
    (I:164; R:  0; G: 38; B:153), (I:165; R: 76; G: 95; B:153),
    (I:166; R:  0; G: 31; B:127), (I:167; R: 63; G: 79; B:127),
    (I:168; R:  0; G: 19; B: 76), (I:169; R: 38; G: 47; B: 76),
    (I:170; R:  0; G:  0; B:255), (I:171; R:127; G:127; B:255),
    (I:172; R:  0; G:  0; B:204), (I:173; R:102; G:102; B:204),
    (I:174; R:  0; G:  0; B:153), (I:175; R: 76; G: 76; B:153),
    (I:176; R:  0; G:  0; B:127), (I:177; R: 63; G: 63; B:127),
    (I:178; R:  0; G:  0; B: 76), (I:179; R: 38; G: 38; B: 76),
    (I:180; R: 63; G:  0; B:255), (I:181; R:159; G:127; B:255),
    (I:182; R: 51; G:  0; B:204), (I:183; R:127; G:102; B:204),
    (I:184; R: 38; G:  0; B:153), (I:185; R: 95; G: 76; B:153),
    (I:186; R: 31; G:  0; B:127), (I:187; R: 79; G: 63; B:127),
    (I:188; R: 19; G:  0; B: 76), (I:189; R: 47; G: 38; B: 76),
    (I:190; R:127; G:  0; B:255), (I:191; R:191; G:127; B:255),
    (I:192; R:102; G:  0; B:204), (I:193; R:153; G:102; B:204),
    (I:194; R: 76; G:  0; B:153), (I:195; R:114; G: 76; B:153),
    (I:196; R: 63; G:  0; B:127), (I:197; R: 95; G: 63; B:127),
    (I:198; R: 38; G:  0; B: 76), (I:199; R: 57; G: 38; B: 76),
    (I:200; R:191; G:  0; B:255), (I:201; R:223; G:127; B:255),
    (I:202; R:153; G:  0; B:204), (I:203; R:178; G:102; B:204),
    (I:204; R:114; G:  0; B:153), (I:205; R:133; G: 76; B:153),
    (I:206; R: 95; G:  0; B:127), (I:207; R:111; G: 63; B:127),
    (I:208; R: 57; G:  0; B: 76), (I:209; R: 66; G: 38; B: 76),
    (I:210; R:255; G:  0; B:255), (I:211; R:255; G:127; B:255),
    (I:212; R:204; G:  0; B:204), (I:213; R:204; G:102; B:204),
    (I:214; R:153; G:  0; B:153), (I:215; R:153; G: 76; B:153),
    (I:216; R:127; G:  0; B:127), (I:217; R:127; G: 63; B:127),
    (I:218; R: 76; G:  0; B: 76), (I:219; R: 76; G: 38; B: 76),
    (I:220; R:255; G:  0; B:191), (I:221; R:255; G:127; B:223),
    (I:222; R:204; G:  0; B:153), (I:223; R:204; G:102; B:178),
    (I:224; R:153; G:  0; B:114), (I:225; R:153; G: 76; B:133),
    (I:226; R:127; G:  0; B: 95), (I:227; R:127; G: 63; B:111),
    (I:228; R: 76; G:  0; B: 57), (I:229; R: 76; G: 38; B: 66),
    (I:230; R:255; G:  0; B:127), (I:231; R:255; G:127; B:191),
    (I:232; R:204; G:  0; B:102), (I:233; R:204; G:102; B:153),
    (I:234; R:153; G:  0; B: 76), (I:235; R:153; G: 76; B:114),
    (I:236; R:127; G:  0; B: 63), (I:237; R:127; G: 63; B: 95),
    (I:238; R: 76; G:  0; B: 38), (I:239; R: 76; G: 38; B: 57),
    (I:240; R:255; G:  0; B: 63), (I:241; R:255; G:127; B:159),
    (I:242; R:204; G:  0; B: 51), (I:243; R:204; G:102; B:127),
    (I:244; R:153; G:  0; B: 38), (I:245; R:153; G: 76; B: 95),
    (I:246; R:127; G:  0; B: 31), (I:247; R:127; G: 63; B: 79),
    (I:248; R: 76; G:  0; B: 19), (I:249; R: 76; G: 38; B: 47),
    (I:250; R: 51; G: 51; B: 51), (I:251; R: 91; G: 91; B: 91),
    (I:252; R:132; G:132; B:132), (I:253; R:173; G:173; B:173),
    (I:254; R:214; G:214; B:214), (I:255; R:255; G:255; B:255));

var
  Acad: OleVariant;
  
implementation

resourcestring
  rsAcadNotFound = 'AutoCAD  !';
  rsAutoCADRegPath = '\SOFTWARE\Autodesk\AutoCAD';

procedure AcConnect(const acClassName: String; const acNewInstance: Boolean;
    const acVisible: Boolean);
var
  IU: IUnknown;
  ClassID: TCLSID;
  IsCreate: Boolean;
  ErrorCode: HResult;
begin
  if VarIsClear(Acad) then //     
  begin
    //  AutoCAD     
    ErrorCode:= CLSIDFromProgID(PWideChar(WideString(acClassName)), ClassID);
    if not Succeeded(ErrorCode) then
      raise EOleSysError.Create(rsAcadNotFound, ErrorCode, 0);
    //       AutoCAD
    IsCreate:= acNewInstance or (not Succeeded(GetActiveObject(ClassID, nil, IU)));
    if IsCreate then
      try
        //    AutoCAD
        Acad:= CreateComObject(ClassID) as IDispatch;
      except
        raise EOleSysError.Create(rsAcadNotFound, ErrorCode, 0);
      end
    else
      Acad:= IU as IDispatch;
    //   AutoCAD
    if not VarIsClear(Acad) then Acad.Visible:= acVisible;
  end;
end;

procedure AcShow(WndState: AcWindowState = acMax);
begin
  if not VarIsClear(Acad) then
  begin
    Acad.Visible:= True;
    Acad.WindowState:= WndState;
  end;
end;

procedure AcHide;
begin
  if not VarIsClear(Acad) then Acad.Visible:= False;
end;

procedure AcRelease;
begin
  if not VarIsClear(Acad) then
  begin
    Acad.Quit;
    Acad:= Unassigned;
  end;
end;

function EnumAutoCAD(Items: TStrings): Boolean;
var
  reg: TRegistry;
begin
  reg:= TRegistry.Create;
  try
    with reg do
    begin
      RootKey:= HKEY_LOCAL_MACHINE;
      Result:= OpenKeyReadOnly(rsAutoCADRegPath);
      if not Result then Exit;
      GetKeyNames(Items);
    end;
  finally
    FreeAndNil(reg);
  end;
end;

function GetCurrAutoCADClassName: String;
var
  reg: TRegistry;
  res: Boolean;
begin
  Result:= EmptyStr;
  reg:= TRegistry.Create;
  try
    with reg do
    begin
      RootKey:= HKEY_CURRENT_USER;
      res:= OpenKeyReadOnly('\Software\Classes\' + rsDefClassName + '\CurVer');
      if not res then Exit;
      Result:= ReadString('');
    end;
  finally
    FreeAndNil(reg);
  end;
end;

function AutoCADFamilyNameToClassName(const AFamilyName: String): String;
var
  p: Integer;
begin
  Result:= AFamilyName;
  p:= Pos('.', Result);
  if Copy(Result, p, MaxInt) = '.0' then
    Delete(Result, p, MaxInt);
  Result:= StringReplace(Result, 'R', rsDefClassName + '.', [rfIgnoreCase]);
end;

function EnumAutoCADHardcopy(Items: TStrings): Boolean;
var
  reg: TRegistry;
begin
  reg:= TRegistry.Create;
  try
    with reg do
    begin
      RootKey:= HKEY_LOCAL_MACHINE;
      Result:= OpenKeyReadOnly('SOFTWARE\Autodesk\Hardcopy');
      if not Result then Exit;
      GetValueNames(Items);
    end;
  finally
    FreeAndNil(reg);
  end;
end;

function EnumAutoCADDocuments(Items: TStrings): Integer;
var
  i, cnt: Integer;
  ActiveDocName, DocName: String;
begin
  Result:= -1;
  if not VarIsClear(Acad) then
  begin
    ActiveDocName:= Acad.ActiveDocument.Name;
    cnt:= Acad.Documents.Count;
    with Items do
    begin
      BeginUpdate;
      try
        Clear;
        for i:= 0 to cnt - 1 do
        begin
          DocName:= Acad.Documents.Item(i).Name;
          Add(DocName);
          if AnsiCompareText(DocName, ActiveDocName) = 0 then
            Result:= i;
        end;
      finally
        EndUpdate;
      end;
    end;
  end;
end;

function EnumAutoCADLayers(Items: TStrings): Integer;
var
  AcadDocument, Layer: OleVariant;
  ActiveLayerName, LayerName: String;
  i, LayersCount, Tmp: Integer;
  States: TLayerStates;
begin
  Result:= -1;
  if not VarIsClear(Acad) then
  begin
    AcadDocument:= Acad.ActiveDocument;
    with Items do
    begin
      BeginUpdate;
      try
        Clear;
        ActiveLayerName:= AcadDocument.ActiveLayer.Name;
        LayersCount:= AcadDocument.Layers.Count;
        for i:= 0 to LayersCount - 1 do
        begin
          Layer:= AcadDocument.Layers.Item(i);
          States:= [];
          if Layer.LayerOn then States:= States + [lsLayerOn];
          if Layer.Freeze then States:= States + [lsFreeze];
          if Layer.Lock then States:= States + [lsLock];
          if Layer.Plottable then States:= States + [lsPlottable];
          LayerName:= Layer.Name;
          System.Move(States, Tmp, 1);
          AddObject(LayerName, TObject(Tmp));
          if AnsiCompareText(LayerName, ActiveLayerName) = 0 then
            Result:= i;
        end;
      finally
        EndUpdate;
      end;
    end;
  end;
end;

function EnumAutoCADTextStyles(Items: TStrings): Integer;
var
  AcadDocument: OleVariant;
  ActiveStyleName, StyleName: String;
  i, StylesCount: Integer;
begin
  Result:= -1;
  if not VarIsClear(Acad) then
  begin
    AcadDocument:= Acad.ActiveDocument;
    with Items do
    begin
      BeginUpdate;
      try
        Clear;
        ActiveStyleName:= AcadDocument.ActiveTextStyle.Name;
        StylesCount:= AcadDocument.TextStyles.Count;
        for i:= 0 to StylesCount - 1 do
        begin
          StyleName:= AcadDocument.TextStyles.Item(i).Name;
          Add(StyleName);
          if AnsiCompareText(StyleName, ActiveStyleName) = 0 then
            Result:= i;
        end;
      finally
        EndUpdate;
      end;
    end;
  end;
end;

function EnumAutoCADDimStyles(Items: TStrings): Integer;
var
  AcadDocument: OleVariant;
  ActiveStyleName, StyleName: String;
  i, StylesCount: Integer;
begin
  Result:= -1;
  if not VarIsClear(Acad) then
  begin
    AcadDocument:= Acad.ActiveDocument;
    with Items do
    begin
      BeginUpdate;
      try
        Clear;
        ActiveStyleName:= AcadDocument.ActiveDimStyle.Name;
        StylesCount:= AcadDocument.DimStyles.Count;
        for i:= 0 to StylesCount - 1 do
        begin
          StyleName:= AcadDocument.DimStyles.Item(i).Name;
          Add(StyleName);
          if AnsiCompareText(StyleName, ActiveStyleName) = 0 then
            Result:= i;
        end;
      finally
        EndUpdate;
      end;
    end;
  end;
end;

function EnumAutoCADLinetypes(Items: TStrings): Integer;
var
  AcadDocument: OleVariant;
  ActiveLinetype, LinetypeName: String;
  i, LinetypesCount: Integer;
begin
  Result:= -1;
  if not VarIsClear(Acad) then
  begin
    AcadDocument:= Acad.ActiveDocument;
    with Items do
    begin
      BeginUpdate;
      try
        Clear;
        ActiveLinetype:= AcadDocument.ActiveLinetype.Name;
        LinetypesCount:= AcadDocument.Linetypes.Count;
        for i:= 0 to LinetypesCount - 1 do
        begin
          LinetypeName:= AcadDocument.Linetypes.Item(i).Name;
          Add(LinetypeName);
          if AnsiCompareText(LinetypeName, ActiveLinetype) = 0 then
            Result:= i;
        end;
      finally
        EndUpdate;
      end;
    end;
  end;
end;

function EnumBlockNames(Items: TStrings; IncludeSpecial: Boolean = False): Integer;
var
  i, BlocksCount: Integer;
  BlkName: string;
begin
  Result:= -1;
  if not VarIsClear(Acad) then
  begin
    with Items do
    begin
      BeginUpdate;
      try
        Clear;
        BlocksCount:= Acad.ActiveDocument.Blocks.Count;
        for i:= 0 to BlocksCount - 1 do
        begin
          BlkName:= Acad.ActiveDocument.Blocks.Item(i).Name;
          if (not IncludeSpecial) and (BlkName[1] <> '*') or IncludeSpecial then
            Add(BlkName);
        end;
      finally
        EndUpdate;
      end;
      Result:= Count;
    end;
  end;
end;

function LayerExist(const ALayerName: String): Boolean;
var
  AcadDocument: OleVariant;
  i, cnt: Integer;
begin
  Result:= False;
  if not VarIsClear(Acad) then
  begin
    AcadDocument:= Acad.ActiveDocument;
    cnt:= AcadDocument.Layers.Count;
    for i:= 0 to cnt - 1 do
    begin
      if AcadDocument.Layers.Item(i).Name = ALayerName then
      begin
        Result:= True;
        Break;
      end;
    end;
  end;
end;

function LinetypeExist(const ALinetypeName: String): Boolean;
var
  AcadDocument: OleVariant;
  i, cnt: Integer;
begin
  Result:= False;
  if not VarIsClear(Acad) then
  begin
    AcadDocument:= Acad.ActiveDocument;
    cnt:= AcadDocument.Linetypes.Count;
    for i:= 0 to cnt - 1 do
    begin
      if AcadDocument.Linetypes.Item(i).Name = ALinetypeName then
      begin
        Result:= True;
        Break;
      end;
    end;
  end;
end;

procedure SetActiveLayer(AName: String);
var
  AcadDocument, Layer: OleVariant;
  ActiveLayerName: String;
  i, LayersCount: Integer;
begin
  if not VarIsClear(Acad) then
  begin
    AcadDocument:= Acad.ActiveDocument;
    ActiveLayerName:= AcadDocument.ActiveLayer.Name;
    if AnsiCompareText(AName, ActiveLayerName) = 0 then Exit;
    LayersCount:= AcadDocument.Layers.Count;
    for i:= 0 to LayersCount - 1 do
    begin
      Layer:= AcadDocument.Layers.Item(i);
      if AnsiCompareText(Layer.Name, AName) = 0 then Break;
    end;
    AcadDocument.ActiveLayer:= Layer;
  end;
end;

function Point3D(X, Y: Double; Z: Double = 0): TPoint3D;
begin
  Result.X:= X;
  Result.Y:= Y;
  Result.Z:= Z;
end;

function AcadPoint(X, Y: Double; Z: Double = 0): OleVariant;
begin
  Result:= VarArrayCreate([0, 2], varDouble);
  Result[0]:= X;
  Result[1]:= Y;
  Result[2]:= Z;
end;

function AcadPoint(APoint: TPoint3D): OleVariant; overload;
begin
  Result:= VarArrayCreate([0, 2], varDouble);
  Result[0]:= APoint.X;
  Result[1]:= APoint.Y;
  Result[2]:= APoint.Z;
end;

function AcadColorToColor(AColor: AcColor): TColor;
begin
  if AColor in [(Low(AcadColorIndex) + 1)..High(AcadColorIndex)] then
    Result:= RGB(AcadColorIndex[AColor].R, AcadColorIndex[AColor].G, AcadColorIndex[AColor].B)
  else
    Result:= clWhite;
end;

function IsBit(const Value: LongWord; const n: TLongWordSize): Boolean;
begin
  Result:= (Value and (1 shl n)) <> 0;
end;

procedure SetBit(var Value: LongWord; const n: TLongWordSize);
begin
  Value:= Value or (1 shl n);
end;

procedure ResetBit(var Value: LongWord; const n: Byte);
begin
  Value:= Value and not (1 shl n);
end;

procedure ExtractBlkRefAttr(ABlkRef: OleVariant; AFileName: string);
var
  AttrArray, AttrRef: OleVariant;
  i, LBnd, HBnd: Integer;
  lst: TStringList;
begin
  if ABlkRef.HasAttributes then
  begin
    lst:= TStringList.Create;
    try
      lst.Add(Format('[%s]', [ABlkRef.Name]));
      AttrArray:= ABlkRef.GetAttributes;
      LBnd:= VarArrayLowBound(AttrArray, 1);
      HBnd:= VarArrayHighBound(AttrArray, 1);
      for i:= LBnd to HBnd do
      begin
        AttrRef:= AttrArray[i];
        lst.Add(Format('%s=%s', [AttrRef.TagString, AttrRef.TextString]));
      end;
      lst.SaveToFile(AFileName);
    finally
      lst.Free;
    end;
  end else
    raise Exception.CreateFmt(' "%s"   ', [ABlkRef.Name]);
end;

procedure ExtractBlkRefAttr(ABlkRef: OleVariant; Items: TStrings);
var
  AttrArray, AttrRef: OleVariant;
  i, LBnd, HBnd: Integer;
begin
  if ABlkRef.HasAttributes then
  begin
    Items.Add(Format('[%s]', [ABlkRef.Name]));
    AttrArray:= ABlkRef.GetAttributes;
    LBnd:= VarArrayLowBound(AttrArray, 1);
    HBnd:= VarArrayHighBound(AttrArray, 1);
    for i:= LBnd to HBnd do
    begin
      AttrRef:= AttrArray[i];
      Items.Add(Format('%s=%s', [AttrRef.TagString, AttrRef.TextString]));
    end;
  end else
    raise Exception.CreateFmt(' "%s"   ', [ABlkRef.Name]);
end;

procedure ExtractBlockAttr(AFileName: string; Space: TWorkSpace = wsModelSpace);
var
  BlkRef: OleVariant;
  BlkName: string;
  i, ObjCount: Integer;
  lst: TStringList;
begin
  if not VarIsClear(Acad) then
  begin
    lst:= TStringList.Create;
    try
      if Space = wsModelSpace then
        ObjCount:= Acad.ActiveDocument.ModelSpace.Count
      else
        ObjCount:= Acad.ActiveDocument.PaperSpace.Count;
      for i:= 0 to ObjCount - 1 do
      begin
        if Space = wsModelSpace then
          BlkRef:= Acad.ActiveDocument.ModelSpace.Item(i)
        else
          BlkRef:= Acad.ActiveDocument.PaperSpace.Item(i);
        if BlkRef.ObjectName = 'AcDbBlockReference' then
        begin
          if BlkRef.HasAttributes then
          begin
            ExtractBlkRefAttr(BlkRef, lst);
          end;
        end;
      end;
      lst.SaveToFile(AFileName);
    finally
      lst.Free;
    end;
  end;
end;

end.
 